package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.service.SavedSearchesService;
import at.ac.uibk.gitsearch.web.rest.errors.BadRequestAlertException;
import at.ac.uibk.gitsearch.service.dto.SavedSearchesDTO;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.PaginationUtil;
import io.github.jhipster.web.util.ResponseUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;
import java.util.stream.StreamSupport;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * REST controller for managing {@link at.ac.uibk.gitsearch.domain.SavedSearches}.
 */
@RestController
@RequestMapping("/api")
public class SavedSearchesResource {

    private final Logger log = LoggerFactory.getLogger(SavedSearchesResource.class);

    private static final String ENTITY_NAME = "savedSearches";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SavedSearchesService savedSearchesService;

    public SavedSearchesResource(SavedSearchesService savedSearchesService) {
        this.savedSearchesService = savedSearchesService;
    }

    /**
     * {@code POST  /saved-searches} : Create a new savedSearches.
     *
     * @param savedSearchesDTO the savedSearchesDTO to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new savedSearchesDTO, or with status {@code 400 (Bad Request)} if the savedSearches has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/saved-searches")
    public ResponseEntity<SavedSearchesDTO> createSavedSearches(@Valid @RequestBody SavedSearchesDTO savedSearchesDTO) throws URISyntaxException {
        log.debug("REST request to save SavedSearches : {}", savedSearchesDTO);
        if (savedSearchesDTO.getId() != null) {
            throw new BadRequestAlertException("A new savedSearches cannot already have an ID", ENTITY_NAME, "idexists");
        }
        SavedSearchesDTO result = savedSearchesService.save(savedSearchesDTO);
        return ResponseEntity.created(new URI("/api/saved-searches/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /saved-searches} : Updates an existing savedSearches.
     *
     * @param savedSearchesDTO the savedSearchesDTO to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated savedSearchesDTO,
     * or with status {@code 400 (Bad Request)} if the savedSearchesDTO is not valid,
     * or with status {@code 500 (Internal Server Error)} if the savedSearchesDTO couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/saved-searches")
    public ResponseEntity<SavedSearchesDTO> updateSavedSearches(@Valid @RequestBody SavedSearchesDTO savedSearchesDTO) throws URISyntaxException {
        log.debug("REST request to update SavedSearches : {}", savedSearchesDTO);
        if (savedSearchesDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        SavedSearchesDTO result = savedSearchesService.save(savedSearchesDTO);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, savedSearchesDTO.getId().toString()))
            .body(result);
    }

    /**
     * {@code GET  /saved-searches} : get all the savedSearches.
     *
     * @param pageable the pagination information.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of savedSearches in body.
     */
    @GetMapping("/saved-searches")
    public ResponseEntity<List<SavedSearchesDTO>> getAllSavedSearches(Pageable pageable) {
        log.debug("REST request to get a page of SavedSearches");
        Page<SavedSearchesDTO> page = savedSearchesService.findAll(pageable);
        HttpHeaders headers = PaginationUtil.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
        return ResponseEntity.ok().headers(headers).body(page.getContent());
    }

    /**
     * {@code GET  /saved-searches/:id} : get the "id" savedSearches.
     *
     * @param id the id of the savedSearchesDTO to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the savedSearchesDTO, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/saved-searches/{id}")
    public ResponseEntity<SavedSearchesDTO> getSavedSearches(@PathVariable Long id) {
        log.debug("REST request to get SavedSearches : {}", id);
        Optional<SavedSearchesDTO> savedSearchesDTO = savedSearchesService.findOne(id);
        return ResponseUtil.wrapOrNotFound(savedSearchesDTO);
    }

    /**
     * {@code DELETE  /saved-searches/:id} : delete the "id" savedSearches.
     *
     * @param id the id of the savedSearchesDTO to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/saved-searches/{id}")
    public ResponseEntity<Void> deleteSavedSearches(@PathVariable Long id) {
        log.debug("REST request to delete SavedSearches : {}", id);
        savedSearchesService.delete(id);
        return ResponseEntity.noContent().headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString())).build();
    }

    /**
     * {@code SEARCH  /_search/saved-searches?query=:query} : search for the savedSearches corresponding
     * to the query.
     *
     * @param query the query of the savedSearches search.
     * @param pageable the pagination information.
     * @return the result of the search.
     */
    @GetMapping("/_search/saved-searches")
    public ResponseEntity<List<SavedSearchesDTO>> searchSavedSearches(@RequestParam String query, Pageable pageable) {
        log.debug("REST request to search for a page of SavedSearches for query {}", query);
        Page<SavedSearchesDTO> page = savedSearchesService.search(query, pageable);
        HttpHeaders headers = PaginationUtil.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
        return ResponseEntity.ok().headers(headers).body(page.getContent());
        }
}
